## $Id: browser.py 9785 2012-12-07 08:59:30Z henrik $
##
## Copyright (C) 2012 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
import httplib
import grok
from zope.component import getUtility
from kofacustom.nigeria.interswitch.browser import (
    InterswitchPaymentRequestWebservicePageStudent
    )
from waeup.kofa.browser.layout import KofaPage
from waeup.kofa.interfaces import RETURNING, CLEARED, IKofaUtils
from waeup.kofa.utils.helpers import to_timezone
from waeup.futminna.students.interfaces import ICustomStudentOnlinePayment
from waeup.futminna.interfaces import MessageFactory as _

PRODUCT_ID = '117'
SITE_NAME = 'futminna-kofa.waeup.org'
PROVIDER_ACCT = '0026781725'
PROVIDER_BANK_ID = '31'
PROVIDER_ITEM_NAME = 'BT Education'
INSTITUTION_NAME = 'FUTMinna'
CURRENCY = '566'
GATEWAY_AMT = 300.0
#QUERY_URL = 'https://webpay.interswitchng.com/paydirect/services/TransactionQueryURL.aspx'
#QUERY_URL = 'https://testwebpay.interswitchng.com/test_paydirect/services/TransactionQueryURL.aspx'
POST_ACTION = 'https://webpay.interswitchng.com/paydirect/webpay/pay.aspx'
#POST_ACTION = 'https://testwebpay.interswitchng.com/test_paydirect/webpay/pay.aspx'

HOST = 'webpay.interswitchng.com'
#HOST = 'testwebpay.interswitchng.com'
URL = '/paydirect/services/TransactionQueryWs.asmx'
#URL = '/test_paydirect/services/TransactionQueryWs.asmx'
httplib.HTTPConnection.debuglevel = 0

class InterswitchPageStudent(KofaPage):
    """ View which sends a POST request to the Interswitch
    CollegePAY payment gateway.
    """
    grok.context(ICustomStudentOnlinePayment)
    grok.name('goto_interswitch')
    grok.template('student_goto_interswitch')
    grok.require('waeup.payStudent')
    label = _('Submit data to CollegePAY (Interswitch Payment Gateway)')
    submit_button = _('Submit')
    action = POST_ACTION
    site_name = SITE_NAME
    currency = CURRENCY
    pay_item_id = ''
    product_id = PRODUCT_ID

    def update(self):
        #if self.context.p_state != 'unpaid':
        if self.context.p_state == 'paid':
            self.flash(_("Payment ticket can't be re-send to CollegePAY."))
            self.redirect(self.url(self.context, '@@index'))
            return

        student = self.student = self.context.student
        certificate = getattr(student['studycourse'],'certificate',None)
        self.amount_auth = 100 * self.context.amount_auth
        xmldict = {}
        if certificate is not None:
            xmldict['department'] = certificate.__parent__.__parent__.code
            xmldict['faculty'] = certificate.__parent__.__parent__.__parent__.code
        else:
            xmldict['department'] = None
            xmldict['faculty'] = None
        self.category = getUtility(IKofaUtils).PAYMENT_CATEGORIES[
            self.context.p_category]
        tz = getUtility(IKofaUtils).tzinfo
        self.local_date_time = to_timezone(
            self.context.creation_date, tz).strftime("%Y-%m-%d %H:%M:%S %Z")
        self.site_redirect_url = self.url(self.context, 'request_webservice')
        # Provider data
        xmldict['detail_ref'] = self.context.p_id
        xmldict['provider_acct'] = PROVIDER_ACCT
        xmldict['provider_bank_id'] = PROVIDER_BANK_ID
        xmldict['provider_item_name'] = PROVIDER_ITEM_NAME
        provider_amt = 1500.0
        xmldict['provider_amt'] = 100 * provider_amt

        # Institution data. Account numbers were changed to the new
        # NUBAN 10 digit number system
        xmldict['institution_acct'] = ''
        xmldict['institution_bank_id'] = ''
        xmldict['institution_acct'] = '000000000000'
        xmldict['institution_bank_id'] = '00'
        xmldict['institution_name'] = INSTITUTION_NAME
        
        if self.context.p_category == 'schoolfee':
            xmldict['institution_amt'] = 100 * (
                self.context.amount_auth - provider_amt - GATEWAY_AMT)
            if self.context.student.current_mode in ('pg_ft'):
                self.pay_item_id = "11703"
            elif self.context.student.state == CLEARED and \
                self.context.student.current_level == 100:
                self.pay_item_id = "11700"
            elif self.context.student.state == CLEARED and \
                self.context.student.current_level == 200:
                self.pay_item_id = "11701"
            elif self.context.student.state == CLEARED and \
                self.context.student.current_level == 300:
                self.pay_item_id = "11702"
            elif self.context.student.state == RETURNING and \
                self.context.student.current_level in (100,110):
                self.pay_item_id = "11701"
            elif self.context.student.state == RETURNING and \
                self.context.student.current_level in (200,210):
                self.pay_item_id = "11702"
            elif self.context.student.state == RETURNING and \
                self.context.student.current_level in (300,310):
                self.pay_item_id = "11703"
            elif self.context.student.state == RETURNING and \
                self.context.student.current_level in (400,410,500,510,600):
                self.pay_item_id = "11704"

            if self.context.student.current_mode == 'jm_ft':
                xmldict['institution_acct'] = "000000000000"
                xmldict['institution_bank_id'] = '00'
            elif self.context.student.current_mode == 'pg_ft':
                xmldict['institution_acct'] = "2005910931"
                xmldict['institution_bank_id'] = '8'
            elif self.context.student.state == CLEARED and \
                self.context.student.current_level == 100:
                xmldict['institution_acct'] = "0021030851"
                xmldict['institution_bank_id'] = '31'
            elif self.context.student.state == CLEARED and \
                self.context.student.current_level == 200:
                xmldict['institution_acct'] = "0005646299"
                xmldict['institution_bank_id'] = '47'
            elif self.context.student.state == CLEARED and \
                self.context.student.current_level == 300:
                xmldict['institution_acct'] = "1010500151"
                xmldict['institution_bank_id'] = '117'
            elif self.context.student.state == RETURNING and \
                self.context.student.current_level in (100,110):
                xmldict['institution_acct'] = "0005646299"
                xmldict['institution_bank_id'] = '47'
            elif self.context.student.current_level in (200,210):
                xmldict['institution_acct'] = "1010500151"
                xmldict['institution_bank_id'] = '117'
            elif self.context.student.current_level in (300,310):
                xmldict['institution_acct'] = "2005910931"
                xmldict['institution_bank_id'] = '8'
            elif self.context.student.current_level in (400,410,500,510,600):
                xmldict['institution_acct'] = "0027490487"
                xmldict['institution_bank_id'] = '10'

        elif self.context.p_category == 'clearance':
            xmldict['institution_amt'] = 100 * (
                self.context.amount_auth - GATEWAY_AMT)
            xmldict['institution_acct'] = "1750005063"
            xmldict['institution_bank_id'] = '120'
            self.pay_item_id = "11706"
            provider_amt = 0.0

        elif 'maintenance' in self.context.p_category:
            xmldict['institution_amt'] = 100 * (
                self.context.amount_auth - GATEWAY_AMT)
            xmldict['institution_acct'] = "2018856637"
            xmldict['institution_bank_id'] = '8'
            self.pay_item_id = "11705"
            provider_amt = 0.0

        # Interswitch amount is not part of the xml data
        if self.context.p_category == 'schoolfee':
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="School Fee" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
<item_detail item_id="2" item_name="%(provider_item_name)s" item_amt="%(provider_amt)d" bank_id="%(provider_bank_id)s" acct_num="%(provider_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        elif self.context.p_category == 'clearance':
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="Acceptance Fee" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict

        elif 'maintenance' in self.context.p_category:
            xmltext = """<payment_item_detail>
<item_details detail_ref="%(detail_ref)s" college="%(institution_name)s" department="%(department)s" faculty="%(faculty)s">
<item_detail item_id="1" item_name="Hostel Maintenance Fee" item_amt="%(institution_amt)d" bank_id="%(institution_bank_id)s" acct_num="%(institution_acct)s" />
</item_details>
</payment_item_detail>""" % xmldict


        self.xml_data = """<input type="hidden" name="xml_data" value='%s'  />""" % xmltext
        self.context.provider_amt = provider_amt
        self.context.gateway_amt = GATEWAY_AMT
        return

class InterswitchPaymentRequestWebservicePageStudent(
    InterswitchPaymentRequestWebservicePageStudent):
    """ Request webservice view for the CollegePAY gateway
    """
    grok.context(ICustomStudentOnlinePayment)
    product_id = PRODUCT_ID
    gateway_host = HOST
    gateway_url = URL
